// Local headers
#include "PixelWriter.h"
#include "font.h"


bool PixelWriter::ClearLEDs()
{
  if (device_==0) return false;
  for (unsigned int i=0;i<8;i++) 
    for (unsigned int j=0;j<8;j++) 
      device_->SetPixel(i,j,SenseHat::BLACK);
  return true;
}


bool PixelWriter::DisplayPattern(const std::vector< std::vector<bool> >& pixels)
{
  if (device_==0) return false;

  // Safety
  if (pixels.size()>8)
  {
    std::cout << "PixelWriter: ERROR - height is too large" << std::endl;
    return false;
  }
  for (unsigned int i=0;i<pixels.size();i++)
  {
    if (pixels[i].size()>8)
    {
      std::cout << "PixelWriter: ERROR - width is too large" << std::endl;
      return false;
    }
  }

  // SetPixel
  for (unsigned int i=0;i<pixels.size();i++)
    for (unsigned int j=0;j<pixels[i].size();j++)
    {
      if (pixels[i][j]) device_->SetPixel(i,j,forecolor_);
      else device_->SetPixel(i,j,backcolor_);
    }

  return true;
}


bool PixelWriter::DisplayPattern(bool pixels[8][8])
{
  if (device_==0) return false;

  // SetPixel
  for (unsigned int i=0;i<8;i++)
    for (unsigned int j=0;j<8;j++)
    {
      if (pixels[i][j]) device_->SetPixel(i,j,forecolor_);
      else device_->SetPixel(i,j,backcolor_);
    }
    
  return true;
}


bool PixelWriter::ConvertCharacterToPattern(char c, std::vector<std::vector<bool> >& pattern)
{
  if (device_==0) return false;

  int icharacter = -1;
  for (unsigned int i=0;i<128;i++)
  {
    if (c==font[i].character)
    {
      icharacter = i;
      break;
    }
  }

  if (icharacter==-1)
  {
    std::cout << "PixelWriter Error: the character '" << c << "' is not found" << std::endl;
    return false;
  }

  pattern.resize(8,std::vector<bool> (8,false));
  for (unsigned int i=0;i<pattern.size();i++)
    for (unsigned int j=0;j<pattern.size();j++)
    {
      if (font[icharacter].pattern[i][j]) pattern[i][j]=true;
    }

  return true;

}

bool PixelWriter::DisplayCharacter(char c)
{
  if (device_==0) return false;

  std::vector< std::vector<bool> > pattern;
  if (!ConvertCharacterToPattern(c,pattern)) return false;
  DisplayPattern(pattern);
  return true;
}


// Display a sentence
bool PixelWriter::DisplaySentence(const std::string& sentence, unsigned int timing)
{
  if (device_==0) return false;
  if (sentence=="") return Clear();

  // Creating the full pattern
  std::vector< std::vector<bool> > pattern(8, std::vector<bool> (0,false));
  std::vector< std::vector<bool> > tmp;

  for (unsigned int i=0;i<sentence.size();i++)
  {
    tmp.clear();
    if (!ConvertCharacterToPattern(sentence[i],tmp)) return false;

    // determining first column
    unsigned int firstcolumn=0;
    for (unsigned int k=0;k<8;k++)
    {
      bool emptycolumn=true;
      for (unsigned l=0;l<8;l++)
      {
	if (tmp[l][k])
	{
	  emptycolumn=false;
	  break;
	}
      }
      if (!emptycolumn)
      {
        firstcolumn=k;
        break;
      }
    }

    // determining last column
    unsigned int lastcolumn=7;
    for (int k=7;k>=0;k--)
    {
      bool emptycolumn=true;
      for (unsigned l=0;l<8;l++)
      {
	if (tmp[l][k])
	{
	  emptycolumn=false;
	  break;
	}
      }
      if (!emptycolumn)
      {
        lastcolumn=k;
        break;
      }
    }


    if (sentence[i]==' ') { firstcolumn=0; lastcolumn=2; }

    // extending the pattern matrix
    unsigned int initialsize = pattern[0].size();
    for (unsigned int k=0;k<8;k++)
    {
      pattern[k].resize(pattern[k].size()+(lastcolumn-firstcolumn+2));
    }

    // Add empty column
    for (unsigned int k=0;k<8;k++)
    {
      pattern[k][initialsize]=false;
    }
    
    // copy
    for (unsigned int k=0;k<8;k++)
      for (unsigned int l=firstcolumn;l<=lastcolumn;l++)
      {
        pattern[k][initialsize+1+l-firstcolumn]=tmp[k][l];
      }
  }

  // 
  tmp.resize(8, std::vector<bool> (8,false));
  for (unsigned int i=0;i<pattern[0].size();i++)
  {
    for (unsigned k=0;k<8;k++)
      for (unsigned l=0;l<8;l++)
      {
	if ((l+i)>=pattern[0].size()) tmp[k][l]=false;
	else tmp[k][l]=pattern[k][l+i];
      }
    DisplayPattern(tmp);
    device_->Sleep(timing);
  }

  return true;
}
