#ifndef PIXEL_WRITER_H
#define PIXEL_WRITER_H


// STL headers
#include <iostream>
#include <vector>

// Lib
#include "SenseHat.h"


class PixelWriter
{
 private :
 
  SenseHat* device_;            /// Link to SenseHat board
  SenseHat::Color_t forecolor_; /// Text color
  SenseHat::Color_t backcolor_; /// Background color
  
  /// Converting a charact to Pattern
  bool ConvertCharacterToPattern(char c, std::vector<std::vector<bool> >& pattern);
  
 public :

  /// Constructor with no argument
  PixelWriter()
  { device_ = 0; forecolor_=SenseHat::BLACK; backcolor_=SenseHat::BLACK; }

  /// Destructor
  ~PixelWriter()
  { }  

  /// Initialize
  PixelWriter(SenseHat* device, SenseHat::Color_t forecolor, SenseHat::Color_t backcolor)
  { device_ = device; forecolor_=forecolor; backcolor_=backcolor; }

  /// Setting the link to the Sense Hat board
  void SetDevice(SenseHat* device)
  { device_ = device; }

  /// Setting the text color
  void SetForecolor(SenseHat::Color_t forecolor)
  { forecolor_ = forecolor; }
  
  /// Setting the background color
  void SetBackcolor(SenseHat::Color_t backcolor)
  { backcolor_ = backcolor; }
    
  /// Clear
  bool Clear()
  { device_ = 0; forecolor_=SenseHat::BLACK; backcolor_=SenseHat::BLACK; }
  
  /// Clear LEDs
  bool ClearLEDs();

  /// Display a pattern
  bool DisplayPattern(const std::vector< std::vector<bool> >& pixels);

  /// Display a pattern
  bool DisplayPattern(bool pixels[8][8]);

  /// Display a character
  bool DisplayCharacter(char c);

  /// Display a sentence
  bool DisplaySentence(const std::string& sentence, unsigned int timing=200000);

};

#endif
