#ifndef STATISTICS_CALCULATOR_H
#define STATISTICS_CALCULATOR_H


// STL headers
#include <vector>
#include <iostream>
#include <string>
#include <cmath>


class StatisticsCalculator
{
 private :

  unsigned int nmax_;         /// number max of data to store
  std::vector<double> data_;  /// collection of data
  bool ordered_;              /// tag to know if the collection is already sorted
  
 public :

  /// Constructor with no argument
  StatisticsCalculator()
  { Clear(); }

  /// Destructor
  ~StatisticsCalculator()
  { }  

  /// Clear
  void Clear()
  {
    nmax_=100000;
    ordered_=false;
    data_.clear();
  }

  /// Print
  void Print()
  {
    std::cout << "nb events: " << data_.size() << std::endl;
    std::cout << "events: | ";
    for(unsigned int i=0;i<data_.size();i++) std::cout << data_[i] << " | ";
    std::cout << std::endl;
  }

  /// Clear
  bool AddValue(double value);

  /// GetMean
  double GetMean()
  { 
    double sumx=0;
    for (unsigned int i=0;i<data_.size();i++) sumx+=data_[i];
    return sumx/static_cast<double>(data_.size()); 
  }

  /// GetRMS
  double GetRMS()
  { 
    double sumx2=0;
    for (unsigned int i=0;i<data_.size();i++) sumx2+=data_[i]*data_[i];  
    return std::sqrt(sumx2/static_cast<double>(data_.size()));
  }

  /// Get Standard Deviation
  double GetStandardDeviation()
  { 
    double sumx=0; double sumx2=0;
    for (unsigned int i=0;i<data_.size();i++)
	{
	  sumx+=data_[i];
	  sumx2+=data_[i]*data_[i];
    }
    return std::sqrt( sumx2/static_cast<double>(data_.size()) - 
                      (sumx/static_cast<double>(data_.size()))*(sumx/static_cast<double>(data_.size())));
  }


   /// Get Mediane
  double GetMediane();

  /// Get Maximum
  double GetMax();

  /// Get Minimum
  double GetMin();
  
  /// Read a CSV file and extract the data from the a given column
  bool ReadCSVfile(const std::string& filename, char column);

};

#endif
